/**
   File name: factory.h
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662

   File description: The following file simulates a factory that
   creates entities. Each of the derived class is created in a
   "production line", whre the derived class is inited with no params
   and they are added in a manner which complies with the inheritance.
   The factory knows all the params that are supposed to be given, as
   does any assembly line.
   The factory uses a ptr to function map in order to determine which
   entity will be created in the factory. These creator methods are
   used to connect the factory with the ctors, and that's why
   they are not factory methods.
*/

#ifndef _FACTORY_H
#define _FACTORY_H

/*-------------------- include files --------------------------------*/
#include <iostream.h>
#include <map>
#include <string>
#include <typeinfo>
#include <strstream>
#include "parser.h"
#include "illegalFormatException.h"
#include "illegalEntityException.h"
#include "illegalParamException.h"
#include "entity.h"
#include "movable.h"
#include "movableEntity.h"
#include "plantFamily.h"
#include "student.h"
#include "teacher.h"
#include "seed.h"
#include "flower.h"
#include "mole.h"
#include "world.h"
#include "mapReader.h"

/* upper hierarchy creators */

/* sets the fields of the newly created entity with the ones
   common to all entities. recieves the factory which is now creating
   the entity. */
void entityCreator(Entity* ent, unsigned int theOrder,
		   MapReader& read);

/* sets the fields of the newly created entity with the ones
       common to all movableEntitys. recieves the factory which is now creating
       the movableEntity. */
void movableEntityCreator(MovableEntity* mover, MapReader& read);

/* sets the fields of the newly created entity with the ones
   common to all plant famiy members. recieves the factory which is now
   creating the plant family member. */
void plantCreator(PlantFamily* pla, MapReader& read);

/* base hierarchy creators */
/* creates student in the given factory. calls the creators of the
   entity and movableEntity. */
Entity* createStudent(unsigned int theOrder, MapReader& read);

/* creates teacher in the given factory. calls the creators of the
   entity and movableEntity. */
Entity* createTeacher(unsigned int theOrder, MapReader& read);

/* creates flower in the given factory. calls the creators of the
   entity and plantFamily. */
Entity* createFlower(unsigned int theOrder, MapReader& read);

/* creates flower-seed in the given factory. calls the creators of the
   entity and plantFamily.*/
Entity* createSeed(unsigned int theOrder, MapReader& read);

/* creates mole-cricket in the given factory. calls the creators of the
   entity and movableEntity.*/
Entity* createMole(unsigned int theOrder, MapReader& read);

/************************ class Factory ******************************/
class Factory{
 public:
  /*-------------------- typedefs -------------------------------------*/
  typedef Entity* (*PtrToFun)(unsigned int, MapReader& read);
  typedef map<string,PtrToFun> funMap;
  
  /*---------------------------- ctor -------------------------------*/
  /* constructs the factory which will create all the entities.
     The constructor inits the function map that consists of an entity
     type name and and a ptr to function to the function which creates
     that type.
  */
  Factory();
  
  /*---------------------------- dtor -------------------------------*/
  ~Factory(){}
  
  /* This method returns a new entity, created on the heap. The init
     function searches for the entity type name in the given string map,
     and then finds the appropriate creator function for that type in
     the function map. It then uses that function to create the new
     entity. */
  Entity* init(strMap s, unsigned int theOrder);

  // consts denoting num of params needed for each entity
  static const unsigned int _STUD_NUM_PARAMS;
  static const unsigned int _TEACH_NUM_PARAMS;
  static const unsigned int _FLOW_NUM_PARAMS;
  static const unsigned int _SEED_NUM_PARAMS;
  static const unsigned int _MOLE_NUM_PARAMS;
  
 private:
  
  // the function map containing the entity type name and their creator
  // functions.
  funMap _functions;
};
#endif

/************************************************************************
				  EOF
*************************************************************************/

