/**
   File name: movable.h
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662

   File description: Defines a movable object.
   a movable can move, and defines the properties moving-time and
   moving-pattern.
*/

#ifndef _MOVABLE_H
#define _MOVABLE_H

/*-------------------------- included files -------------------------*/
#include <iostream.h>
#include <string>
#include "checks.h"
#include "point.h"
#include "illegalEntityException.h"

class Movable{
 public:
      /*------------------------- ctors -----------------------------*/

      /* a default constructor. */
      Movable(): _randomMove(false),_currentIndex(0){}

      /* constructs a movable, defining the movement fields. */
      Movable(float moveTime, const string movePattern);
      
      /*--------------------- dtor -----------------------------------*/
      virtual ~Movable(){}

      /*----------------- accessory methods --------------------------*/

      /* gets the moving time of the movable. */
      float getMoveTime()const {return(_moveTime);}

      /* gets the moving pattern of the movable. */
      string getMovePattern()const {return(_movePattern);}

      /* sets the movement time of the movable. */
      void setMoveTime(float moveTime);

       /* sets the movement pattern of the movable.*/
      void setMovePattern(string movePattern);

      /* returns the next moving direction of the movable. */
      char getNextDirection();

      const Point getNextLocation(unsigned int x, unsigned int y, char nextDir);

      // consts for the string repres. of the fields of the movable
      static const string _MOVE_TIME;
      static const string _MOVE_PAT;
      
      // const string representing the movements of the movable..
      static const string _RANDOM;
      static const char* _DIRECTIONS;
      
 protected:
      // denotes the movement-time of the movable.
      float _moveTime;
      
      // denotes the movement-pattern of the movable.
      string _movePattern;

      // denotes whether the entity moves in a random pattern
      bool _randomMove;

      // represents the current index of the movement pattern
      unsigned int _currentIndex;

      /* checks if the given string represents a legal movement of
	 the movable object. Returns true if the move is legal,
	 false otherwise. */
      bool legalMove(string str);


      // enum for the chars repres. the direction of the movement.
      enum DirectionChars{NORTH = 'n', SOUTH = 's', EAST = 'e', WEST = 'w',
			  NUM_DIRECTIONS = 4};

      /* returns the next point to which the movable will move. */
      //Point getNextLocation(); // this is in MovableEntity
};

#endif

/***********************************************************************
			       EOF
************************************************************************/
