/**
   File name: world.h
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
   
   File description: this class describes a world in the simulation. It acts
   as the collection of all databases linked to the simulation and its 
   occurences. 
   The main DBs are:
   * Map connecting between the ptrs to the entities and their order in the 
   simulation.
   * Location map which links between a location and the orders of the 
   entities currently in that location.
   * Dead entity queue - collects the dead entities after a round of 
   encounters, is emptied after a set of encounters whithin a location.
   
   The world also includes the wind, which moves entites which aren't
   self moving.   
*/

#ifndef _WORLD_H
#define _WORLD_H

#include <map>
#include <queue>
#include <string>
#include "wind.h"
#include "grid.h"
#include "mapReader.h"
#include "movableEntity.h"
#include "encounterManager.h"

class World{
 public:
      typedef Grid<unsigned int> Grid;
      typedef map<unsigned int, Entity*> EntityMap;
      typedef queue<unsigned int> Queue;

      // iterators on the entity list data base
      typedef EntityMap::const_iterator const_iterator; 
      typedef EntityMap::iterator iterator; 

      // iterators to a specified list in the grid
      typedef Grid::const_iterator const_location_iterator;
      typedef Grid::iterator location_iterator;

      // iterators on the whole grid data base
      typedef Grid::const_grid_iterator const_world_iterator;
      typedef Grid::grid_iterator world_iterator;

      World(){}

      World(unsigned int xSize, unsigned int ySize, float movingTime, 
	    const string movingPattern):
	    _xSize(xSize), _ySize(ySize),_wind(movingTime,movingPattern){} 
      
      ~World(){}

      /* Returs the Grid of the world, which is a map linking a 2DPoint
	 to a list of the orders of the entities.*/
      Grid& getGrid() {return _grid;}

      EntityMap& getEntityMap() {return _entMap;}

      /* Returns a pointer to an entity which is identified by his order
	 in the EntityMap. Returns null if the order given is not in the
	 EntityMap. */
      Entity* getEntity(unsigned int order)const;

      Wind& getWind() {return _wind;}

      /* adds an entity to this world */
      void addEntity(Entity* ent);

      /* removes an entity from this world */
      void removeEntity(unsigned int order);
      
      /* moves the entity in the world and encounters with the entities 
	 in the new location. */
      void moveAndEncounter(MovableEntity* ent, EncounterManager& manager);

      /////////// add wrapper method to allow movement of seeds!!!!!!!!!!!
	//////////////////////
	void changePoint(MovableEntity* ent);
	
	/* the given entity will encounter all of the entites in its target
	   position (which is his current x, y positions). */
	void encounterAll(Entity* ent, EncounterManager& manager);
	
	/* adds an entity to a given cell, using his xpos and ypos as the
	   target point. */
	void addToPoint(Entity* ent);
	
	const_location_iterator begin(Entity* ent){
	  return(_grid.begin(Point(ent->getXPos(),ent->getYPos())));
	}
	
	const_location_iterator end(Entity* ent){
	  return(_grid.end(Point(ent->getXPos(),ent->getYPos())));
	}
	
	/*   const_location_iterator begin(const Point& p){ */
	/* 	return(_grid.begin(p)); */
	/*       } */
	
	/*       const_location_iterator end(const Point& p){ */
	/* 	return(_grid.end(p)); */
	/*       } */
	
	/* returns the x-axis size of the world. */ 
	unsigned int getXSize()const {return _xSize;}
	
	/* returns the y-axis size of the world. */ 
	unsigned int getYSize()const {return _ySize;}

	/* adds the order representing the entity into the dead-entities 
	   queue and alters the entity's aliveness flag. */ 
	void kill(Entity* ent){
	      _tempDead.push(ent->getOrder());
	      ent->setAliveness(false);   
	}
	
	/* adds the order representing the entity into the dead-entities 
       queue and alters its aliveness flag. */
	//void kill(unsigned int order);{
	    //	    Entity* ent = getEntity(order);
	    //	    ent->setAliveness(false);
	    //	    _tempDead.push(order);
	    //      }
      
      /* cleans the database of the dead people */
      void buryAll();

      /* adds the order of a new flying entity into the flyers DB,
       located in the wind object */
      //      void addFlyer(unsigned int order){_wind.addFlyer(order);}
       
      /* removes the order of a flying entity from the flyers DB,
       located in the wind object */
      //      void removeFlyer(unsigned int order){_wind.removeFlyer(order);}
      
      // deldeldel ???????????
      /* returns the representation of the world as an ostream, along with
        the point of the position of each entity. */
      ostream& printWorld(ostream& out = cout);

      /* dumps the list of entites as described in the ex description */
      ostream& dump(ostream& out = cout);

      // denotes the minimal x, y values in the world.
      static const int _MIN_COORD;

      //      flyers_iterator begin() {return _wind.begin();}

      //      flyers_iterator end() {return _wind.end();}

      //      float getMoveTime()const {return _wind.getMoveTime();}

 private:
      // denotes the sizes of the world, x is south to north, 
      // y is east to west.
      unsigned int _xSize;
      unsigned int _ySize;

      Wind _wind;
      Grid _grid;
      EntityMap _entMap;

      // the queue containing th orders of all the entities which have been 
      // killed in an encounter
      Queue _tempDead;
};
#endif

/*********************************************************************
			       EOF
**********************************************************************/
